package Engine::Actions::AdminInstall;
use strict;

use XFileConfig;
use Engine::Core::Action ( 'IMPLEMENTS' => [qw(install)] );

use JSON;
use LWP::UserAgent;
use Installer;
use CheckParams;

use File::Slurp;
use File::Basename;

sub main {
   my $ua = LWP::UserAgent->new(agent => "XFS-UpdateAgent");
   my @packages = $ses->require("UpdatesChecker")->checkUpdates();
   $ses->PrintTemplate("admin_install.html",
      packages => \@packages,
      number_to_update => int(grep { !$_->{up_to_date} } @packages),
      updates => JSON::encode_json({ map { $_->{package_id} => $_ } @packages }),
      cliid => $ses->{cliid},
   );
}

sub install {
   my $pkg_context = { product => 'xfspro', license => $c->{license_key}, package_id => $f->{package_id} };
   my $result = $f->{srv_id} ? remote_install($f->{srv_id}, $pkg_context) : local_install($pkg_context);
   return $ses->PrintJSON($result) if $result->{status} ne 'OK';

   use Data::Dumper;
   print STDERR "result = ", Dumper($result), "\n";

   register_package({
      srv_id => $f->{srv_id} || 0,
      package_id => $result->{package_id},
      package_version => $result->{package_version},
   });

   return $ses->PrintJSON($result);
}

sub local_install {
   my ($pkg_context) = @_;

   my $installer = Installer->new({
      checksum_checker => \&local_checksum_checker,
      deploy_option => 'MAIN',
   });
   
   my $ret = $installer->deploy($pkg_context, { cgi_bin => $c->{cgi_path}, html => $c->{site_path} });
   return $ret if $ret->{status} ne 'OK';
   die("No pkg_path") if !$ret->{pkg_path};

   Installer::run_install_script("$ret->{pkg_path}/install.pl") if -e "$ret->{pkg_path}/install.pl";
   process_install_sql("$ret->{pkg_path}/install.sql") if -e "$ret->{pkg_path}/install.sql";

   my $updater = $ses->require("UpdatesChecker");
   my @remaining_servers = grep
      { Engine::Components::UpdatesChecker::cmp_versions($ret->{package_version}, $_->{installed_version}||'0.0' ) > 0 }
      @{ $db->SelectARef("SELECT * FROM PkgVersions WHERE package_id=?", $ret->{package_id}) };

   return $ret;
}

sub remote_install {
   my ($srv_id, $pkg_context) = @_;
   my $srv = $db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $f->{srv_id});
   return { error => "No such file server" } if !$srv;

   print STDERR "Deploying $pkg_context->{package_id} to $srv->{srv_name} (srv_id = $srv->{srv_id})...\n";

   my $res = $ses->api2($srv->{srv_id}, {
      op => 'install',
      %$pkg_context
   });
   
   print STDERR "Got from file server: $res\n";
   return JSON::decode_json($res);
}

sub local_checksum_checker {
   my ($package_id, $checksums) = @_;

   my %ret = map
      { $_ => $db->SelectOne("SELECT COUNT(*) FROM PkgFiles WHERE package_id=? AND rel_path=? AND md5=?", $package_id, $_, $checksums->{$_}) }
      keys(%$checksums);

   return \%ret;
}

sub register_package {
   my ($opts) = @_;
   check_params($opts, qw(*package_id *srv_id *package_version));

   $db->Exec("INSERT INTO PkgVersions SET package_id=?, srv_id=?, installed_version=?, last_updated=NOW()
      ON DUPLICATE KEY UPDATE installed_version=?, last_updated=NOW()",
      $opts->{package_id},
      $opts->{srv_id} || 0,
      ($opts->{package_version}) x 2);
}

sub process_install_sql {
   my ($sql_path) = @_;
   print STDERR "Applying install.sql\n";

   my $rules = read_file($sql_path);

   for(grep { length($_) > 3 } split(';', $rules)) {
      eval { $db->Exec($_) };
      progress({ status => "ERROR", type => "SQL", message => $@ }) if $@;
   }
}

1;
