package Engine::Actions::Payments;
use strict;

use XFileConfig;
use Engine::Core::Action (
   'IMPLEMENTS' => [qw(checkout check_transaction)],
   'ANTICSRF_WHITELIST' => [qw(checkout)]
);
use JSON;
use HCE_MD5;
use XUtils;

sub main
{
   return $ses->redirect( $c->{site_url} ) unless $c->{enabled_prem};
   return checkout() if $f->{amount}; # Legacy templates support

   if ( my $file = $db->SelectRow( "SELECT * FROM Files WHERE file_code=?", $ses->getEnv('HTTP_REFERER') =~ /\/(\w{12})/ ) )
   {
      $ses->setCookie( "aff", $file->{usr_id}, '+14d' );
   }

   my @payment_types = $ses->require("PaymentAcceptor")->getAvailablePaymentMethods();
   my $limits = $ses->require("PremiumComparison")->mkPremiumComparison();

   require Time::Elapsed;
   my $et               = new Time::Elapsed;
   my @plans            = @{ $ses->ParsePlans( $c->{payment_plans}, 'array' ) };
   my @traffic_packages = @{ $ses->ParsePlans( $c->{traffic_plans}, 'array' ) };
   for ( @plans, @traffic_packages )
   {
      $_->{payment_types} = \@payment_types;
   }

   $ses->PrintTemplate(
      "payments.html",
      %{$limits},
      payment_types  => \@payment_types,
      plans          => \@plans,
      premium        => $ses->getUser && $ses->getUser->{premium},
      expire_elapsed => $ses->getUser && $et->convert( $ses->getUser->{exp_sec} ),
      'rand'         => $ses->randchar(6),
      ask_email      => $ses->{utype} eq 'anon' && !$c->{no_anon_payments},
      monthprice     => compute_monthly_price(@plans),
      'currency_symbol' => ( $c->{currency_symbol} || '$' ),
      'traffic_packages' => \@traffic_packages,
      'token_payments'   => $ses->genToken(op => 'payments'),
   );
}

sub checkout
{
   if ( $c->{no_anon_payments} && !$ses->getUser )
   {
      my $next = { op => 'payments', 'amount' => $f->{amount}, type => $f->{type} };
      $next->{target} = $f->{target} if $f->{target};
      $next->{submethod} = $f->{submethod} if $f->{submethod};

      my $hce = HCE_MD5->new($c->{dl_key}, "XFileSharingPRO");
      my $next_base32 = $ses->encode32($hce->hce_block_encrypt(encode_json($next)));
      return $ses->redirect("$c->{site_url}/register?next=$next_base32");
   }

   $f->{amount} = sprintf( "%.02f", $f->{amount} );
   ($f->{type}, $f->{submethod}) = split(/-/, $f->{type}, 2) if $f->{type} =~ /-/;

   return $ses->message($ses->{lang}->{lang_not_reseller}) if $c->{m_k_manual} && $f->{reseller} && !$ses->getUser->{usr_reseller};
   $f->{referer} = 'RESELLER' if $f->{reseller};

   my %opts = %{$f};
   $opts{target} = $f->{target} || 'premium_days';
   $opts{target} = 'reseller' if $f->{reseller};

   $opts{usr_id} = $ses->getUser ? $ses->getUserId : 0;
   $opts{email} = $ses->getUser->{usr_email} if $ses->getUser;
   $opts{days}    = $ses->ParsePlans( $c->{payment_plans}, 'hash' )->{ $f->{amount} } if $opts{target} eq 'premium_days';
   $opts{traffic} = $ses->ParsePlans( $c->{traffic_plans}, 'hash' )->{ $f->{amount} } if $opts{target} eq 'premium_traffic';
   $opts{aff_creds} = $ses->require("AffTracker")->get();

   $opts{product_code} = "$opts{target}_$opts{amount}";
   $opts{product_name} = {
      'premium_days' => "$opts{days} days premium account",
      'premium_traffic' => "$opts{traffic} GB of premium traffic",
      'reseller' => "$f->{amount} $c->{currency_code} of reseller funds",
   }->{$opts{target}};

   if($f->{target} =~ /^file_(\d+)_access$/)
   {
      my $file = $db->SelectRow("SELECT * FROM Files WHERE file_id=?", $1);
      my $owner = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $file->{usr_id}) if $file;
      return $ses->message("No such file or not a VIP file") if !$file || !$ses->require("DlTracker")->isVipFile($file);
      return $ses->message($ses->{lang}->{lang_invalid_payment_amount}) if $file->{file_price} != $f->{amount};

      $opts{aff_creds}->{aff_id} = $owner->{usr_id};
      $opts{aff_creds}->{file_id} = $file->{file_id};
      $opts{product_name} = "$file->{file_name} access";
   }

   if($opts{target} eq 'reseller') {
      $opts{aff_id} = 0;
   }

   my $settings = $db->SelectRow("SELECT * FROM PaymentSettings WHERE name=?", $f->{type});
   $opts{amount} = sprintf("%0.2f", $opts{amount} / (1 - $settings->{commission} / 100)) if $settings && $settings->{commission_mode} eq 'TAKE_FROM_BUYER';

   return $ses->message($ses->{lang}->{lang_invalid_payment_amount}) if $opts{target} eq 'premium_days'    && !$opts{days};
   return $ses->message($ses->{lang}->{lang_invalid_payment_amount}) if $opts{target} eq 'premium_traffic' && !$opts{traffic};

   my $paymentAcceptor = $ses->require("PaymentAcceptor");
   my $transaction = $paymentAcceptor->createTransaction(map { ($_ => $opts{$_} ) } qw(usr_id amount traffic days email target type aff_creds));
   $opts{id} = $transaction->{id};
   $opts{email} = $ses->getUser->{usr_email} if $ses->getUser;
   $ses->setCookie( 'transaction_id', $transaction->{id} );

   my ($plg) = grep { $_->options()->{name} eq $f->{type} } $ses->getPlugins('Payments');
   return $ses->message($ses->{lang}->{lang_no_appropriate_plugin}) if !$plg;

   my $options = $plg->options();
   my ($submethod) = grep { $_->{name} eq $f->{submethod} } @{ $options->{submethods} } if $options->{submethods};
   return $ses->message("Unknown submethod: $f->{submethod}") if $options->{submethods} && !$submethod;

   my $url = $plg->checkout(\%opts);

   # Some APIs aren't allowing to pass the transaction ID with Return URL
   return $ses->redirect($url) if $url =~ /^http/ && !$f->{no_redirect};
}

sub check_transaction
{
   my $transaction = $db->SelectRow("SELECT * FROM Transactions WHERE id=?", $f->{check_transaction});
   return $ses->PrintJSON({ status => 'verified' }) if $transaction && $transaction->{verified};
   return $ses->PrintJSON({ status => 'unknown' });
}

sub compute_monthly_price
{
   my @sorted = sort { abs($a->{day}-30) <=> abs($b->{day}-30) } @_;
   return sprintf("%d", 30 * $sorted[0]->{amount} / $sorted[0]->{days}) if @sorted;
}

1;
