package Engine::Components::CRUDService;
use strict;
use vars qw($ses $db $c $f);
use JSON;
use XUtils;

sub new {
   my ($class, $table) = @_;
   my $self = { table => $table };

   $self->{fields} = $db->SelectARef("DESCRIBE `$self->{table}`");
   $self->{display_filters} = {};
   $self->{actions} = {};
   $self->{show} = {};
   $self->{titles} = {};
   $self->{constraints} = {};
   $self->{known_fields}->{$_->{Field}} = $_ for @{ $self->{fields} };
   $self->{default_sort_field} = $self->{fields}->[0]->{Field};

   my @key = map { $_->{Field} } grep { $_->{Key} eq 'PRI' } @{ $self->{fields} };
   die("Table $self->{sql_table} has no primary key") if !@key;
   $self->{key} = \@key;

   bless $self, $class;

   $self->actions(qw(create edit delete));
   $self->show(map { $_->{Field} } grep { $self->{Extra} eq '' } @{ $self->{fields} });

   return $self;
}

sub actions {
   my ($self, @list) = @_;

   for(@list) {
      die("Unknown default action: $_") if !/^(create|edit|delete)$/;
   }

   $self->{actions} = { map { $_ => 1 } @list };

   return $self;
}

sub show {
   my ($self, @list) = @_;

   for(@list) {
      die("Unknown field: $_") if ! $self->{known_fields}->{$_};
   }

   $self->{show} = { map { $_ => 1 } @list };

   return $self;
}

sub title {
   my ($self) = @_;
   $self->{titles}->{$_[1]} = $_[2] if @_ == 3;
   $self->{title} = $_[1] if @_ == 2;
   return $self;
}

sub constraint {
   my ($self, $field, $value) = @_;
   $self->{constraints}->{$field} = $value;
   $self->{show}->{$field} = 0;
   return $self;
}

sub display_filter($$&) {
   my ($self, $keys, $value) = @_;
   for(@{ XUtils::ARef($keys) }) {
      $self->{display_filters}->{$_} = $value;
   }
   return $self;
}

sub serve {
   my ($self) = @_;
   return $self->edit() if $f->{edit} || $f->{create};
   return $self->save() if $f->{save};
   return $self->delete() if $f->{delete};
   $self->findAll();
}

### List

sub findAll {
   my ($self) = @_;

   ## Paging & sorting
   $f->{sort_field} ||= $self->{default_sort_field};
   $f->{sort_order} ||= $self->{default_sort_order} || 'down';
   my $sort = $ses->require("Sorting")->makeSortSQLcode( $self->{default_sort_field} );
   my %sort = $ses->require("Sorting")->makeSortHash([ $self->_visibleColumns() ]);
   my $paging = $ses->makePagingSQLSuffix( $f->{page} );

   my @columns = map { $self->_makeColumn($_) } $self->_visibleColumns();

   for(@columns) {
      $_->{sort} = $sort{"s_$_->{name}"};
      $_->{sort2} = $sort{"s2_$_->{name}"};
      $_->{value} = $f->{"filter.$_->{name}"};
      $_->{is_0} = 1 if $_->{value} eq '0';
      $_->{is_1} = 1 if $_->{value} eq '1';
   }

   ## Filtering
   my %filters = (
      ( map { /\.([a-zA-Z0-9-_]+)$/; $1 => $f->{$_} } grep { /^filter\./ && $f->{$_} ne '' } keys(%$f) ),
      %{ $self->{constraints} || {} },
   );

   my $filters = $self->_makeFilters(%filters);
   $filters = "WHERE $filters" if $filters;

   ## DB interaction
   my $list = $db->SelectARef("SELECT * FROM `$self->{table}` $filters $sort $paging");
   my $total = $db->SelectOne("SELECT COUNT(*) FROM `$self->{table}` $filters");
   my @rows = map { $self->_makeRow($_) } @$list;

   my @row_actions = ();
   push @row_actions, { name => 'edit', title => 'Edit' } if $self->{actions}->{edit};
   $_->{actions} = \@row_actions for @rows;

   my @bulk_actions = ();
   push @bulk_actions, { name => 'create', title => 'Create' } if $self->{actions}->{create};

   if($self->{actions}->{delete}) {
      my $delete_action = { name => 'delete', title => 'Delete' };
      push @row_actions, $delete_action;
      push @bulk_actions, $delete_action;
   }

   $ses->PrintTemplate("crud_list.html",
      op => $f->{op},
      title => $self->{title} || $self->{table},

      column_count => int(@columns) + int(@row_actions) + 1,
      columns => \@columns,

      rows_count => int(@rows),
      rows => \@rows,

      actions_count => int(@row_actions),
      bulk_actions => \@bulk_actions,

      paging       => $ses->makePagingLinks( $f, $total ),
      );
}

sub _makeRow {
   my ($self, $db_row) = @_;
   my @columns = map { { db_column => $_, value => $db_row->{$_} } } $self->_visibleColumns();

   for my $column (@columns) {
      $column->{value} = $column->{value} ? "Yes" : "No" if $self->_columnType($column->{db_column}) eq 'tinyint';
      my $display_filter = $self->{display_filters}->{$column->{db_column}};
      if($display_filter) {
         $_ = $column->{value};
         $column->{value} = $display_filter->($db_row);
      }
   }

   return {
      columns => \@columns,
      actions => [],
      key => $self->_makeKey($db_row),
   }
}

sub _makeKey {
   my ($self, $db_row) = @_;

   my @key_fields = @{ $self->{key} };

   return @key_fields <= 1
      ? $db_row->{$key_fields[0]}
      : JSON::encode_json([ map { $db_row->{$_} } @key_fields ]);
}

sub _makeColumn {
   my ($self, $db_column) = @_;

   my $row = { name => $db_column, title => $self->{titles}->{$db_column} || ucfirst($db_column) };
   $row->{is_flag} = 1 if $self->_columnType($row->{name}) eq 'tinyint';
   return $row;
}

sub _visibleColumns {
   my ($self) = @_;
   return map { $_->{Field} } grep { $self->{show}->{$_->{Field}} } @{ $self->{fields} };
}

sub _columnType {
   my ($self, $db_column) = @_;
   my $column_info = $self->{known_fields}->{$db_column} || return "";
   my $type = $column_info->{Type};
   $type =~ s/\(.*//;
   return $type;
}

sub _makeFilters {
   my ($self, %filters) = @_;

   my @filters = map {
      $self->_columnType($_) =~ /int/i
         ? "`$_`=" . int($filters{$_})
         : "`$_` LIKE " . $db->master()->dbh()->quote("%$filters{$_}%");
   } keys(%filters);

   return @filters ? join(" AND ", @filters) : "";
}

### Create/Edit/Update

sub edit {
   my ($self) = @_;
   my ($filter, @key_values) = $self->_makePrimaryFilter($f->{key});

   my $constraints = $self->_makeFilters(%{ $self->{constraints} });
   $constraints = "AND $constraints" if $constraints;

   my $result = $f->{key} eq ''
      ? [ { map { $_->{Field} => $_->{Default} || '' } @{ $self->{fields} } } ]
      : $db->SelectARef("SELECT * FROM `$self->{table}` WHERE $filter $constraints", @key_values);

   return $ses->message("Item not found: $f->{key}") if !@$result;
   return $ses->message("Key ambiguity detected: $f->{key}") if @$result > 1;

   my @fields = map { { name => $_, title => $self->{titles}->{$_} || ucfirst($_), value => $result->[0]->{$_} } } $self->_visibleColumns();
   for(@fields) {
      $_->{type} = 'checkbox' if $self->_columnType($_->{name}) eq 'tinyint';
      $_->{type} = 'textarea' if $self->_columnType($_->{name}) eq 'text';
      $_->{type} ||= 'text';
      $_->{"is_$_->{type}"} = 1;

      $_->{value} = $db->SelectOne("SELECT CURRENT_TIMESTAMP") if $self->_columnType($_->{name}) eq 'timestamp' && uc($_->{value}) eq 'CURRENT_TIMESTAMP';
   }

   return $ses->PrintTemplate("crud_edit.html", 
      op => $f->{op},
      fields => \@fields,
      key => $f->{key},
      action_edit => 1);
}

sub save {
   my ($self) = @_;
   return $ses->message("Anti-CSRF check failed while saving an item") if !$ses->checkToken();

   my ($filter, @key_values) = $self->_makePrimaryFilter($f->{key});

   for(keys(%{ $self->{constraints} })) {
      $f->{$_} = $self->{constraints}->{$_};
   }
   
   my @fields_to_update = $self->_visibleColumns();
   my $fields_to_update = join(", ", map { "$_=?" } @fields_to_update);
   my @values = map { $f->{"item.$_"} || '' } @fields_to_update;

   if($f->{key} eq '') {
      $db->Exec("INSERT INTO `$self->{table}` SET $fields_to_update", @values);
   } else {
      $db->Exec("UPDATE `$self->{table}` SET $fields_to_update WHERE $filter", @values, @key_values);
   }

   return $ses->redirect("$c->{site_url}/?op=$f->{op}");
}

sub _makePrimaryFilter {
   my ($self, $key) = @_;
   my @key_fields = @{ $self->{key} };
   my @key_values = $key =~ /^\[/ ? @{ JSON::decode_json($ses->UnsecureStr($key)) } : $key;
   my $filter = join(" AND ", map { "$_=?" } @key_fields);
   return ($filter, @key_values);
}

## Delete

sub delete {
   my ($self) = @_;
   return $ses->message("Anti-CSRF check failed while deleting an item") if !$ses->checkToken();

   for my $key_txt(@{ XUtils::ARef($f->{key}) }) {
      my ($filter, @key_values) = $self->_makePrimaryFilter($key_txt);
      $db->Exec("DELETE FROM `$self->{table}` WHERE $filter", @key_values);
   }

   return $ses->redirect("$c->{site_url}/?op=$f->{op}");
}

1;
