package Engine::Components::UsersRegistry;
use strict;
use vars qw($ses $db $c $f);
use CheckParams;

use PBKDF2::Tiny;
use MIME::Base64;

sub getUserById
{
   my ($self, $usr_id) = @_;

   return $db->SelectRow("SELECT *, UNIX_TIMESTAMP(usr_premium_expire)-UNIX_TIMESTAMP() as exp_sec,
         (SELECT SUM(f.file_size) FROM Files f WHERE f.usr_id=u.usr_id) as total_size
         FROM Users u
         WHERE u.usr_id=?",
         $usr_id)
}

sub createUser
{
   my ($self, $opts) = @_;
   check_params($opts, qw(*login email password premium_days security_lock status aff_id pay_email pay_type profit_mode notes phone social social_id));

   my $passwd_hash = $self->genPasswdHash($opts->{password} || $ses->randchar(32));

   $db->Exec(
      "INSERT INTO Users 
              SET usr_login=?, 
                  usr_email=?, 
                  usr_password=?,
                  usr_created=NOW(),
                  usr_premium_expire=NOW()+INTERVAL ? DAY,
                  usr_security_lock=?,
                  usr_status=?,
                  usr_aff_id=?,
                  usr_pay_email=?, 
                  usr_pay_type=?,
                  usr_profit_mode=?,
                  usr_notes=?,
                  usr_phone=?,
                  usr_social=?,
                  usr_social_id=?,
                  usr_no_emails=?",
      $opts->{login},
      $opts->{email}||'',
      $passwd_hash,
      $opts->{premium_days}||0,
      $opts->{security_lock}||'',
      $opts->{status}||'OK',
      $opts->{aff_id}||0,
      $opts->{pay_email}||'',
      $opts->{pay_type}||'',
      $opts->{profit_mode}||$c->{m_y_default}||'PPD',
      $opts->{notes}||'',
      $opts->{phone}||'',
      $opts->{social}||'',
      $opts->{social_id}||'',
      $opts->{no_emails}||$c->{gdpr_unsubscribed_default}||0);
    
   return $db->getLastInsertId;
}

sub deleteUser {
   my ($self, $usr_id) = @_;
   $db->Exec("DELETE FROM Users WHERE usr_id=?",$usr_id);
   $db->Exec("DELETE FROM Sessions WHERE usr_id=?",$usr_id);
   $db->Exec("DELETE FROM Payments WHERE usr_id=?",$usr_id);
   $db->Exec("DELETE FROM Transactions WHERE usr_id=?",$usr_id);
   $db->Exec("DELETE FROM Folders WHERE usr_id=?",$usr_id);
   $db->Exec("DELETE FROM IP2RS WHERE usr_id=?",$usr_id);
}

sub randomLogin
{
   my $login = join('', map int rand 10, 1..7);
   while($db->SelectOne("SELECT usr_id FROM Users WHERE usr_login=?",$login)){ $login = join '', map int rand 10, 1..7; }
   return $login;
}

sub setUserPassword
{
   my ($self, $usr_id, $new_password) = @_;
   return if !length($new_password);

   $db->Exec( "UPDATE Users SET usr_password=?, usr_social='' WHERE usr_id=?", $self->genPasswdHash($new_password), $usr_id );
}

sub genPasswdHash
{
   my ($self, $pass) = @_;
   my $turns = 1000;
   my $salt = join('', map { chr( rand(256) ) } (1..24));
   my $data = PBKDF2::Tiny::derive('SHA-256', $pass, $salt, $turns);
   my $hash = sprintf("sha256:%d:%s:%s",
      $turns,
      MIME::Base64::encode_base64($salt, ''),
      MIME::Base64::encode_base64($data, ''));

   return $hash;
}

sub checkForDelayedRedirects
{
   my ($self, $user) = @_;
   my $udata = $ses->getUserData($user->{usr_id}) || return;
   return if !$udata->{delayed_redirect};

   $self->setDelayedRedirect($user->{usr_id}, "");
   return $ses->redirect($udata->{delayed_redirect});
}

sub setDelayedRedirect
{
   my ($self, $usr_id, $url) = @_;
   $ses->setUserData($usr_id, delayed_redirect => $url);
}

sub genFTPPassword
{
   my ($self, $user) = @_;
   require Digest::MD5;
   return uc(Digest::MD5::md5_hex(join(":",
      $user->{usr_id},
      $user->{usr_login},
      $user->{usr_password},
      $c->{dl_key},
      $c->{license_key})));
}

1;
