package S3Utils;
use strict;

use POSIX qw(strftime);
use Digest::SHA qw(sha256_hex hmac_sha256);
use URI;
use URI::Escape;

sub _check(\%\@\@)
{
   my ($opts, $required, $defined) = @_;
   my %valid = map { $_ => 1 } (@$required, @$defined);

   my ($pkg, $fn, $ln) = caller(1);
   for(@$required) { die("Required option: $_ at $fn:$ln\n") if !defined($opts->{$_}); }
   for(keys %$opts) { die("Unknown option: $_ at $fn:$ln\n") if !$valid{$_}; }
}

sub sign_lwp_request {
   my ($creds, $req, %opts) = @_;

   my @time = gmtime();
   my $date = strftime("%Y%m%d", @time);
   my $amz_date = strftime("%Y%m%dT%H%M%SZ", @time);

   my $uri = URI->new($req->uri());
   my $sha256 = $opts{content_sha256}||sha256_hex($req->content);
   $req->header("Host" => $uri->host()) if !$req->header("Host");
   $req->header("x-amz-content-sha256" => $sha256);
   $req->header("x-amz-date" => "$amz_date");

   my %headers;
   $req->headers()->scan(sub { $headers{$_[0]} = $_[1] if lc($_[0]) !~ /^(content-length)/ });

   my $credential = "$creds->{access_key}/$date/$creds->{region}/s3/aws4_request";

   my $canonical_request = mk_canonical_request(method => $req->method(),
      path => $uri->path(),
      query => canonicalize_query_string($uri->query()),
      headers => \%headers,
      content_sha256 => $sha256);

   my $string_to_sign = join("\n",
      "AWS4-HMAC-SHA256",
      "$amz_date",
      "$date/$creds->{region}/s3/aws4_request",
      sha256_hex($canonical_request),
   );

   my $signing_key = S3Utils::signing_key($creds->{secret_key}, $date, $creds->{region});
   my $sign = unpack("H*", hmac_sha256($string_to_sign, $signing_key)); # => aeeed9bbccd4d02ee5c0109b86d86835f995330da4c265957d157751f604d404
   my $signed_headers = join(';', sort(map { lc } keys %headers));

   $req->header('authorization' => "AWS4-HMAC-SHA256 Credential=$credential, SignedHeaders=$signed_headers, Signature=$sign");
}

sub signing_key {
   my ($secret_key, $date, $region) = @_;
   my $date_key = hmac_sha256($date, "AWS4$secret_key");
   my $date_region_key = hmac_sha256($region, $date_key);
   
   my $date_region_service_key = hmac_sha256("s3", $date_region_key);
   my $signing_key = hmac_sha256("aws4_request", $date_region_service_key);
}

sub mk_query_string {
   my (%query) = @_;
   return join("&", map { "$_=".uri_escape($query{$_}) } sort(keys(%query)));
}

sub canonicalize_query_string {
   my ($query) = @_;
   return join("&", map { join("=", /(.*)=?(.*)?/) } split(/&/, $query));
}

sub mk_canonical_request {
   my (%opts) = @_;
   my @required = qw(method path query headers content_sha256);
   my @defined = qw();
   _check(%opts, @required, @defined);

   die("headers should be a hash reference") if ref($opts{headers}) ne 'HASH';

   my @header_names = sort(keys(%{ $opts{headers} }));
   my @canonical_headers = map { lc($_) . ":" . $opts{headers}->{$_} } @header_names;

   return join("\n",
      $opts{method},
      $opts{path},
      $opts{query},
      sort(@canonical_headers),
      "",
      join(";", map { lc } @header_names),
      $opts{content_sha256},
   );
}

1;
