package Plugins::CDN::amazon_s3;
use vars qw($ses $db $c);
use strict;
use URI::Escape qw(uri_escape);
use LWP::UserAgent;
use XML::Simple;
use Digest::SHA qw(sha256_hex hmac_sha256);
use List::Util qw(sum);
use S3Utils;
use POSIX qw(strftime);

sub options {
   return {
      name=>'amazon', title=>'Amazon S3',
      listed => 1,
      s_fields=>[
         {title=>'Access key', name=>'s3_access_key', type=>'text', size=>'32'},
         {title=>'Secret key', name=>'s3_secret', type=>'text', size=>'32'},
         {title=>'Bucket', name=>'s3_bucket', type=>'text', size=>'32'},
         {title=>'Region', name=>'s3_region', type=>'text' },
         ]
      };
}

sub genDirectLink {
   my ($self, $file, %opts) = @_;
   my $server = $ses->db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $file->{srv_id});
   return undef if $server->{srv_cdn} ne 'amazon';

   my $dx = sprintf("%05d",($file->{file_real_id}||$file->{file_id})/$c->{files_per_folder});
   my $resource = "uploads/$dx/$file->{file_real}";

   my $srv_data = $ses->getSrvData($server->{srv_id});
   my $creds = get_creds($srv_data);
   my $fname = uri_escape($opts{fname} || $file->{file_name});

   my @time = gmtime();
   my $date = strftime("%Y%m%d", @time);
   my $amz_date = strftime("%Y%m%dT%H%M%SZ", @time);

   my $credential = "$creds->{access_key}/$date/$creds->{region}/s3/aws4_request";

   my %query = (
      'X-Amz-Algorithm' => 'AWS4-HMAC-SHA256',
      'X-Amz-Credential' => $credential,
      'X-Amz-Date' => $amz_date,
      'X-Amz-Expires' => $c->{symlink_expire} * 3600,
      'X-Amz-SignedHeaders' => 'host',
      'response-content-disposition' => 'attachment; filename="' . $fname . '"',
   );

   my $query = S3Utils::mk_query_string(%query);

   my $canonical_request = S3Utils::mk_canonical_request(method => "GET",
      path => "/$resource",
      query => $query,
      headers => { host => $creds->{host} },
      content_sha256 => 'UNSIGNED-PAYLOAD');

   my $string_to_sign = join("\n",
      "AWS4-HMAC-SHA256",
      "$amz_date",
      "$date/$creds->{region}/s3/aws4_request",
      sha256_hex($canonical_request),
   );

   my $signing_key = S3Utils::signing_key($creds->{secret_key}, $date, $creds->{region});
   my $sign = unpack("H*", hmac_sha256($string_to_sign, $signing_key));

   return "https://$creds->{host}/$resource?$query&X-Amz-Signature=$sign\n";
}

sub delFiles {
   my ($self, $srv_id, $files) = @_;
   my $server = $ses->db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $srv_id);
   return undef if $server->{srv_cdn} ne 'amazon';

   my $srv_data = $ses->getSrvData($server->{srv_id});
   my $creds = get_creds($srv_data);

   for(@$files) {
      next if !$_->{dx} || !$_->{file_real};
      amz_request($creds, DELETE => "https://$creds->{host}/uploads/$_->{dx}/$_->{file_real}");
   }
}

sub runTests {
   my ($self, $f) = @_;
   return undef if $f->{srv_cdn} ne 'amazon';
   my @tests;
   push @tests, 'Max disk usage: ERROR' if !$f->{srv_disk_max} || $f->{srv_disk_max}<=0;

   my $creds = get_creds($f);
   my $ret = amz_request($creds, "GET" => "https://$creds->{host}/");
   push @tests, "Connecting to bucket: ERROR: $ret->{Message}" if $ret->{HttpCode} != 200;

   return(@tests);
}

sub getDiskSpace {
   my ($self, $srv_id) = @_;
   my $srv_data = $ses->getSrvData($srv_id);
   my $creds = get_creds($srv_data);

   my $ret = amz_request($creds, "GET" => "https://$creds->{host}/")->{Contents};
   my @objects = ref($ret) eq 'ARRAY' ? @$ret : $ret;
   my $occupied = sum(map { $_->{Size} } @objects);

   return {
      srv_disk => $occupied,
   }
}

sub get_creds {
   my ($f) = @_;

   return {
      access_key => $f->{s3_access_key},
      secret_key => $f->{s3_secret},
      bucket => $f->{s3_bucket},
      region => $f->{s3_region},
      host => $f->{s3_bucket} =~ /\./
         ? $f->{s3_bucket}
         : "$f->{s3_bucket}.s3.amazonaws.com",
   };
}

sub amz_request {
   my ($creds, $method, $url) = @_;

   my $ua = LWP::UserAgent->new(agent => 'XFileSharingPro');
   my $req = HTTP::Request->new($method => $url);
   S3Utils::sign_lwp_request($creds, $req);

   my $res = $ua->request($req);
   my $ret = $res->content ? XMLin($res->content) : {};
   $ret->{HttpCode} = $res->code;

   return $ret;
}

1;
