package Plugins::CDN::azure;
use vars qw($ses $db $c);
use strict;
use URI::Escape qw(uri_escape);
use Digest::SHA qw(hmac_sha256);
use MIME::Base64 qw(encode_base64 decode_base64);
use Net::Azure::StorageClient;
use POSIX;

sub options {
	return {
		name=>'azure', title=>'Windows Azure',
		listed => 1,
		s_fields=>[
			{title=>'Account Name', name=>'azure_account_name', type=>'text'},
			{title=>'Primary Key', name=>'azure_primary_key', type=>'text', size=>'32'},
			]
		};
}

sub genDirectLink {
	my ($self, $file, %opts) = @_;
	my $server = $ses->db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $file->{srv_id});
	return undef if $server->{srv_cdn} ne 'azure';

	my $srv_data = $ses->getSrvData($server->{srv_id});
	my $dx = sprintf("%05d",($file->{file_real_id}||$file->{file_id})/$c->{files_per_folder});

	# Microsoft expects us to ensure that symlink_expire will not exceed 1h.
	# It's possible to enlarge it up to 10 hours by using a Signed Identifier & ACLs
	# which will complicate the things drastically.
	my $iso_8061 = '%Y-%m-%dT%H:%M:%S.0000000Z';
	my $start_time = strftime($iso_8061, gmtime(time() - 20));
	my $end_time = strftime($iso_8061, gmtime(time() + 1800));
	my $content_disposition = "attachment; filename=$file->{file_name}";

	local *escape = sub { uri_escape($_[0], "=+\/") };
	my $start_time_encoded = escape($start_time);
	my $end_time_encoded = escape($end_time);
	my $content_disposition_encoded = escape($content_disposition);

	my $key = decode_base64($srv_data->{azure_primary_key});
	my $string_to_sign;
	$string_to_sign .= $_ for (
			"r\n",
			"$start_time\n",
			"$end_time\n",
			"/$srv_data->{azure_account_name}/uploads/$dx/$file->{file_real}\n",
			"\n",
			"2013-08-15\n",
			"\n",
			"$content_disposition\n",
			"\n",
			"\n",
			"binary"
		);
	my $sign = escape(encode_base64(hmac_sha256($string_to_sign, $key)));

	return "http://$srv_data->{azure_account_name}.blob.core.windows.net/uploads/$dx/$file->{file_real}?sv=2013-08-15&st=$start_time_encoded&se=$end_time_encoded&sr=b&sp=r&rscd=$content_disposition_encoded&rsct=binary&sig=$sign";
}

sub delFiles {
	my ($self, $srv_id, $files) = @_;
	my $server = $ses->db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $srv_id);
	return undef if $server->{srv_cdn} ne 'azure';

	my $azure = &getAzureClient($srv_id);
	for(@$files) {
		next if !$_->{dx} || !$_->{file_real};
		my $res = $azure->delete_blob("uploads/$_->{dx}/$_->{file_real}");
	}

}

sub runTests {
	my ($self, $f) = @_;
	return undef if $f->{srv_cdn} ne 'azure';
	my @tests;

	push @tests, 'max disk usage: ERROR' if !$f->{srv_disk_max} || $f->{srv_disk_max}<=0;
	
	my $azure = &getAzureClient($f->{srv_id});
	push @tests, $azure->list_containers()->code == 200 ? 'login: OK' : 'login: ERROR';
	return(@tests);
}

sub getAzureClient {
	my ($srv_id) = @_;
	my $srv_data = $ses->getSrvData($srv_id);
	return Net::Azure::StorageClient->new(
                        type => 'Blob',
                        account_name => $srv_data->{azure_account_name},
                        primary_access_key => $srv_data->{azure_primary_key},
                        [ protocol => 'https', ],
                        [ api_version => '2012-02-12', ] )
}

1;
