#!/usr/bin/perl -X
use strict;
use lib ".";
use XFileConfig;
use Session;
use CGI::Carp qw(fatalsToBrowser);
use MIME::Base64;
use URI::Escape qw(uri_unescape);
use XUtils;
use Engine::Extras::VFS;
use Engine::WebDAV::Response::PropStat;
use Data::Dumper;
use Log;

Log->new(filename => 'webdav.log');

my $ses = Session->new;
my $db = $ses->db;
my $f = $ses->f;

method_options() if $ENV{REQUEST_METHOD} eq 'OPTIONS';

my $user = $ses->{user} = &BasicAuth();
&Return(403, "WebDAV access is not enabled for your user type") if ($c->{mod_webdav_reg} || $c->{mod_webdav_prem}) && !$c->{"mod_webdav_$user->{utype}"};

my $vfs = Engine::Extras::VFS->new($ses);

my ($base, $sep, $path) = splitpath($ENV{REQUEST_URI});
$path ||= '/';
$path = uri_unescape($path);

my $subname = 'method_' . lc($ENV{REQUEST_METHOD});
my $sub = \&{ $subname } if defined(&{ $subname });
&Return(501, "Method is not supported: $ENV{REQUEST_METHOD}") if !$sub;

eval {
   $sub->();
};

&Return(500, $@) if $@;

sub Return
{
   my ($code, $msg) = @_;
   print "Status: $code\n";
   print "Content-type: text/plain\n\n";
   print "$msg\n";
   exit();
}

sub PrintXML
{
   my ($code, $content) = @_;
   print "Status: $code\n";
   print "Content-type: text/xml\n\n";
   print '<?xml version="1.0" encoding="utf-8"?>', "\n";
   print $content;
   exit();
}

sub BasicAuth
{
   my ($login, $pass) = split(/:/, decode_base64($1), 2) if $ENV{HTTP_CGI_AUTHORIZATION} =~ /Basic (.*)/;
   my $auth_module = $ses->require("Engine::Components::Auth");
   my $user = $auth_module->checkLoginPass($login, $pass);
   return $user if $user;

   print "Status: 401\n";
   print "WWW-Authenticate: Basic realm=\"XFileSharing\"\n\n";
   exit();
}

sub splitpath
{
   my ($path) = @_;
   return $path =~ /(.*?(webdav.cgi|webdav))(.*)/;
}

sub method_propfind
{
   my $res = Engine::WebDAV::Response::PropStat->new;

   my $file = $vfs->get_file_by_path($path) if $path !~ /\/$/;
   if($file)
   {
      $res->add_file({ path => "$base/$file->{path}", size => $file->{file_size}, created => $file->{file_created} });
      &PrintXML(207, $res->as_string);
   }

   my $dir = $vfs->get_dir_by_path($path);
   &Return(404, "No such path: $path") if !$dir && $path ne '/';

   my $now = $db->SelectOne("SELECT NOW()");

   my ($dirs, $files) = $vfs->list($path, depth => $ENV{HTTP_DEPTH}, include_self => 1);
   $res->add_dir({ path => "$base/$_->{path}", created => $now }) for @$dirs;
   $res->add_file({ path => "$base/$_->{path}", size => $_->{file_size}, created => $_->{file_created} }) for @$files;
   &PrintXML(207, $res->as_string);
}

sub method_proppatch
{
   method_propfind(); # We don't have any useful data to take, so just ignore
}

sub method_head
{
   my $file = $vfs->get_file_by_path($path);
   &Return(404, "No such file: $path") if !$file;

   print "Content-Length: $file->{file_size}\n";
   print "Content-Type: application/octet-stream\n\n";
}

sub method_get
{
   my $file = $vfs->get_file_by_path($path) if $path !~ /\/$/;
   &Return(404, "File not found: $path") if !$file;

   my $srv = $db->SelectRow("SELECT * FROM Servers WHERE srv_id=? AND srv_status!='OFF'", $file->{srv_id}) || &Return(503, "Server not found or offline");
   my $direct_link = $ses->getPlugins('CDN')->genDirectLink({ %$srv, %$file });
   print "Location: $direct_link\n\n";
}

sub method_options
{
   print "DAV: 1,2\n";
   print "DAV: <http://apache.org/dav/propset/fs/1>\n";
   print "MS-Author-Via: DAV\n";
   print "Allow: OPTIONS,GET,HEAD,POST,DELETE,TRACE,PROPFIND,PROPPATCH,COPY,MOVE,LOCK,UNLOCK\n";
   print "Content-Length: 0\n";
   print "Content-Type: httpd/unix-directory\n\n";
}

sub method_put
{
   my $server = $ses->require("Engine::Components::ServersRegistry")->selectServer( $ses->getUser() );

   my $usr_id = $ses->getUserId;
   print STDERR "usr_id=$usr_id\n";
   my $sess = GetSession($usr_id)||StartSession($usr_id);
   $path =~ s/^\/+//;

   my $redirect_url = "$server->{srv_cgi_url}/webdav_put.cgi/$path?sess_id=$sess\n";

   print STDERR "Redirecting to $redirect_url\n";
   print "Status: 301\n";
   print "Location: $redirect_url\n\n";
}

sub method_move
{
   my ($base, $sep, $dest_path) = splitpath($ENV{HTTP_DESTINATION});
   $dest_path = uri_unescape($dest_path);

   &Return(409, "No destination specified") if !$ENV{HTTP_DESTINATION};
   &Return(412, "Can't overwrite $ENV{HTTP_DESTINATION}") if $vfs->exists($dest_path);

   $vfs->move($path, $dest_path);

   print "Status: 201\n";
   print "Location: $ENV{HTTP_DESTINATION}\n\n";
   exit();
}

sub method_delete
{
   $vfs->delete($path);
   print "Status: 204\n\n";
   exit();
}

sub method_mkcol
{
   $vfs->mkdir($path);
   print "Status: 204\n\n";
   exit();
}

sub method_lock
{
   print "Content-Type: text/xml\n\n";
   print <<BLOCK
   <D:prop xmlns:D="DAV:"> 
   <D:lockdiscovery> 
      <D:activelock> 
         <D:locktype>
            <D:write />
         </D:locktype> 
         <D:lockscope>
            <D:exclusive />
         </D:lockscope> 
         <D:depth>infinity</D:depth> 
         <ns0:owner xmlns:ns0="DAV:">
            <ns0:href>Admin-PC\Admin</ns0:href>
         </ns0:owner>
         <D:timeout>Second-3600</D:timeout> 
         <D:locktoken> 
            <D:href>opaquelocktoken:a25014c1-6e33-4418-8012-9a82504638d2</D:href> 
         </D:locktoken> 
      </D:activelock> 
   </D:lockdiscovery> 
</D:prop>
BLOCK
;
}

sub StartSession
{
   my ($usr_id) = @_;
   my $sess_id = $ses->randchar(16);
   $db->Exec("DELETE FROM Sessions WHERE last_time + INTERVAL 5 DAY < NOW()");
   $db->Exec("INSERT INTO Sessions (session_id,usr_id,last_ip,last_useragent,last_time) VALUES (?,?,?,?,NOW())",
      $sess_id,$usr_id,$ses->getIP,$ses->getEnv('HTTP_USER_AGENT')||'');
   return $sess_id;
}

sub GetSession
{
   my ($usr_id) = @_;
   my $session = $db->SelectRow("SELECT * FROM Sessions WHERE usr_id=?", $usr_id);
   return $session ? $session->{session_id} : &StartSession($usr_id);
}
