package Installer;
use strict;
use CheckParams;
use File::Slurp;
use Data::Dumper;
use File::Basename;
use File::Find;
use File::Copy;
use Digest::MD5 qw(md5_hex);
use JSON;

sub new {
   my ($class, $opts) = @_;
   bless copy_options($opts, qw(*checksum_checker *deploy_option)), $class;
}

sub deploy {
   my ($self, $pkg_context, $dirs) = @_;
   check_params($pkg_context, qw(*product *license *package_id));
   check_params($dirs, qw(*cgi_bin *html));

   ## Fetching package
   my $pkg_path = fetch_package($pkg_context);
   return { error => "Failed to get $pkg_context->{package_id}" } if !$pkg_path;
   return { error => "install.json not found" } if ! -e "$pkg_path/install.json";
   return { error => "checksum.jsons not found" } if ! -e "$pkg_path/checksums.json";
   my $manifest = load_json("$pkg_path/install.json");

   ## Preparing deploy list
   my @deploy_list;
   my $pkg_checksums = load_json("$pkg_path/checksums.json");

   for my $file(files_to_deploy("$pkg_path/install.json", $self->{deploy_option})) {
      if($file->{rel_path} =~ m#^(cgi-bin|html)/(.*)#) {
         $file->{dst_path} = sprintf("%s/%s", { "cgi-bin" => $dirs->{cgi_bin}, "html" => $dirs->{html} }->{$1}, $2) || next;
         $file->{pkg_md5} = $pkg_checksums->{$file->{rel_path}};
         $file->{src_md5} = file_md5($file->{src_path});
         $file->{dst_md5} = file_md5($file->{dst_path}) if -e $file->{dst_path} && -s $file->{dst_path};

         return { error => "No checksum found for $file->{rel_path}" } if !$file->{pkg_md5};
         return { error => "Download checksum mismatch for $file->{rel_path}: expected $file->{pkg_md5}, got $file->{src_md5}" }
            if $file->{pkg_md5} ne $file->{src_md5};

         print STDERR "Pushing to deploy list: $file->{src_path}\n";
         push @deploy_list, $file;
      }
   }

   ## Checking that it's ok to rewrite if there are any non-empty files in the destination
   if(my @to_check = grep { $_->{dst_md5} } @deploy_list) {
      my %request = map { $_->{rel_path} => $_->{dst_md5} } @deploy_list;
      my $result = $self->{checksum_checker}->($pkg_context->{package_id}, \%request);

      for(@deploy_list) {
         if(-s $_->{dst_md5} > 0 && $_->{src_md5} ne $_->{dst_md5} && !$result->{$_->{rel_path}}) {
            print STDERR "Refusing to overwrite $_->{dst_path}. Src md5: $_->{src_md5} Dst md5: $_->{dst_md5}\n";
            return { error => "$_->{dst_path} was modified by somebody, refusing to overwrite" };
         }
      }
   }

   ## Actually deploying the files
   for my $file(@deploy_list) {
      my $dir = dirname($file->{dst_path});
      return { error => "Failed to create a directory $dir: $!" } if ! -d $dir && !mkdir($dir, 0755);

      my $ret = move($file->{src_path}, $file->{dst_path});
      return { error => "Failed to write $file->{dst_path}: $!" } if !$ret && ! -f $file->{dst_path};
   }

   return {
      status => 'OK',
      pkg_path => $pkg_path,
      package_id => $manifest->{package_id},
      package_version => $manifest->{package_version},
   };
}

sub find_files {
   my @found;
   find({ wanted => sub { push @found, $_ if -f $_ }, no_chdir => 1 }, @_);
   return @found;
}

sub files_to_deploy {
   my ($path_to_manifest, $mode) = @_;

   die("Wrong files_to_deploy mode: $mode") if $mode !~ /^(MAIN|FS|MAIN\+FS)$/;
   my %flags = map { $_ => 1 } split(/\+/, $mode);

   my $manifest = load_json($path_to_manifest);
   my $pkg_path = dirname($path_to_manifest);
   $pkg_path =~ s/\/+$//;

   my @list; 
   push @list, find_files(map { "$pkg_path/$_" } @{ $manifest->{deploy_to_main} }) if $flags{MAIN} && $manifest->{deploy_to_main};
   push @list, find_files(map { "$pkg_path/$_" } @{ $manifest->{deploy_to_fs} }) if $flags{FS} && $manifest->{deploy_to_fs};

   my @unique_list = keys(%{ { map { $_ => 1 } @list } });

   return grep { defined($_->{rel_path}) }
      map { { src_path => $_, rel_path => substr($_, length($pkg_path) + 1) } }
      @unique_list;
}

sub remove_newlines {
   my ($json) = @_;
   my @lines = split(/\n\r?/, $json);
   s/^\s+//, s/\s+$//, s/\t/ /g for @lines;
   return join('', @lines);
}

sub fetch_package {
   my ($opts) = @_;
   check_params($opts, qw(*license *product *package_id));

   if(! -f '/var/www/mods/'.$opts->{package_id}) {
      print STDERR "Failed to get $opts->{package_id}\n";
      return;
   }

    my $res;
    open(my $fh, '<', '/var/www/mods/'.$opts->{package_id}) or die("Error while negotiating with server\n");
    {
        local $/;
        $res = <$fh>;
    }
    close($fh);

   my $temp_dir = File::Temp::tempdir("install_XXXXXXXXX", CLEANUP => 1, DIR => "temp");
   write_file("$temp_dir/$opts->{package_id}.tgz", $res);
   system("tar -C $temp_dir -xvf $temp_dir/$opts->{package_id}.tgz >/dev/null");
   return $temp_dir;
}

sub run_install_script {
   my ($script_path) = @_;
   my @ret;

   print STDERR "Running $script_path\n";
   do($script_path);
   push @ret, { message => "Failed while running $script_path: $@" } if $@;

   return @ret;
}

sub file_md5 {
   my ($file_path) = @_;
   return md5_hex(read_file($file_path, { binmode => ':raw'}));
}

sub load_json {
   my ($file_path) = @_;
   return read_json(scalar(read_file($file_path)));
}

sub read_json {
   my ($json) = @_;
   return JSON::decode_json(remove_newlines($json));
}

sub remove_newlines {
   my ($json) = @_;
   my @lines = split(/\n\r?/, $json);
   s/^\s+//, s/\s+$//, s/\t/ /g for @lines;
   return join('', @lines);
}

1;
